#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com

import sys
import argparse
import argcomplete
from obsiasset.utils.appConfig import AppConfig
from obsiasset.utils.appManager import AppManager


class OptObj(object):
    def __init__(self):
        self.name = None
        self.metavar = None
        self.action = None
        self.help = None
        self.type = None
        self.choices = None
        self.default = None


class OptItems(object):
    def __init__(self, app_config: AppConfig, app_manager: AppManager):

        self.vault = OptObj()
        self.vault.name = "--vault"
        self.vault.metavar = "<VAULT>"
        self.vault.action = "store"
        self.vault.default = "."
        self.vault.help = "Specify vault directory, if empty it will be set as default value {}".format(self.vault.default)
        self.vault.type = str

        schemas = app_manager.get_supported_schemas()
        self.schema = OptObj()
        self.schema.name = "--schema"
        self.schema.metavar = "<{}>".format("|".join(schemas))
        self.schema.action = "store"
        self.schema.default = ""
        self.schema.help = "Select schema"
        self.schema.type = str
        self.schema.choices = schemas

        languages = app_manager.get_supported_languages()
        self.lang = OptObj()
        self.lang.name = "--lang"
        self.lang.metavar = "<{}>".format("|".join(languages))
        self.lang.action = "store"
        self.lang.default = "en"
        self.lang.help = "Select language"
        self.lang.type = str
        self.lang.choices = languages

        self.sample = OptObj()
        self.sample.name = "--import-sample"
        self.sample.metavar = "<{}>".format("|".join(languages))
        self.sample.action = "store_true"
        self.sample.default = False
        self.sample.help = "Import assets sample"


class AppArgumentParser(argparse.ArgumentParser):
    def error(self, message: str):
        self.print_help(sys.stderr)
        sys.exit(1)


def add_argument_to_sub_parser(sub_parser_item, arg_item, is_required):
    if arg_item.action == "store_true":
        sub_parser_item.add_argument(
            arg_item.name,
            action=arg_item.action,
            help="{} : {}".format(arg_item.help, " | ".join(arg_item.choices)) if arg_item.choices else arg_item.help
        )
        return True
    sub_parser_item.add_argument(
        arg_item.name,
        metavar=arg_item.metavar,
        action=arg_item.action,
        default=arg_item.default,
        help="{} : {}".format(arg_item.help, " | ".join(arg_item.choices)) if arg_item.choices else arg_item.help,
        type=arg_item.type,
        choices=arg_item.choices,
        required=is_required
    )
    return True


class AppParser(object):
    def __init__(self):
        self.app_config = AppConfig()
        self.app_manager = AppManager()
        self.parser = AppArgumentParser(description="")
        argcomplete.autocomplete(self.parser)
        sub_parsers = self.parser.add_subparsers(
            dest="command",
            help=""
        )

        arg_items = OptItems(self.app_config, self.app_manager)

        sub_parsers.add_parser(
            "show-version",
            help="display version info for this tool and your Python runtime"
        )

        # sub_parsers.add_parser(
        #     "show-variables",
        #     help="show all supported environment variables"
        # )

        # sub_parsers.add_parser(
        #     "show-config",
        #     help="show config"
        # )

        sub_parser_init = sub_parsers.add_parser(
            "init",
            help="Init assets vault"
        )
        # add_argument_to_sub_parser(
        #     sub_parser_init,
        #     arg_items.vault,
        #     is_required=False
        # )
        add_argument_to_sub_parser(
            sub_parser_init,
            arg_items.schema,
            is_required=True
        )
        add_argument_to_sub_parser(
            sub_parser_init,
            arg_items.lang,
            is_required=True
        )
        add_argument_to_sub_parser(
            sub_parser_init,
            arg_items.sample,
            is_required=False
        )

        # sub_parser_reorg = sub_parsers.add_parser(
        #     "reorg",
        #     help="Re-organize assets vault"
        # )
        # add_argument_to_sub_parser(
        #     sub_parser_reorg,
        #     arg_items.vault,
        #     is_required=False
        # )
        # add_argument_to_sub_parser(
        #     sub_parser_reorg,
        #     arg_items.schema,
        #     is_required=True
        # )


if __name__ == "__main__":
    print("🚀 This is CLI script")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com


import os
import yaml
import json
from urllib.parse import urlparse, urlunparse
import importlib.util
from importlib.metadata import version, PackageNotFoundError
from obsiasset.utils.appLogger import AppLogger


def clean_url(url):
    parsed = urlparse(url)
    netloc = parsed.netloc.split("@")[-1]
    return urlunparse((
        parsed.scheme,
        netloc,
        parsed.path.rstrip(".git"),
        "",
        "",
        ""
    ))


class AppConfig(object):
    def __init__(self):
        pkg_spec = importlib.util.find_spec("obsiasset")
        self.pkg_dir = os.path.dirname(pkg_spec.origin) if pkg_spec and pkg_spec.origin else None
        self.tmpl_dir = os.path.join(self.pkg_dir, "tmpl") if self.pkg_dir else None
        self.schema_dir = os.path.join(self.pkg_dir, "schema") if self.pkg_dir else None
        self.sample_dir = os.path.join(self.pkg_dir, "sample") if self.pkg_dir else None
        self.i18n_dir = os.path.join(self.pkg_dir, "i18n") if self.pkg_dir else None
        self.app_workdir = os.getcwd()
        self.app_workdir_name = os.path.basename(self.app_workdir)

        self.pkg_name = "obsiasset".replace("-", "_")
        self.cmd_name = "obsiasset"
        self.app_name = self.cmd_name.replace("-", "_")
        self.env_prefix = self.app_name.upper() + "_"

        # self.app_config_file = os.path.join(
        #     os.path.expanduser("~"),
        #     ".{}".format(self.app_name),
        #     "credential.conf"
        # )
        self.app_config_yaml = dict()
        # if os.path.exists(self.app_config_file):
        #     with open(self.app_config_file, "r", encoding="utf-8") as f:
        #         self.app_config_yaml = yaml.safe_load(f.read())
        #     if not self.app_config_yaml:
        #         self.app_config_yaml = dict()

        self.app_logger = AppLogger()

    # def create_config_file(self, config_file_json: dict, force=False):
    #     config_exists = os.path.exists(self.app_config_file)
    #     if config_exists and force:
    #         for config_key, config_value in config_file_json.items():
    #             self.app_config_yaml[config_key] = config_value
    #         with open(self.app_config_file, "w", encoding="utf-8") as f:
    #             f.write(yaml.dump(self.app_config_yaml))
    #     elif not config_exists:
    #         os.makedirs(os.path.join(os.path.expanduser("~"), ".{}".format(self.app_name)), exist_ok=True)
    #         self.app_config_yaml = config_file_json
    #         with open(self.app_config_file, "w", encoding="utf-8") as f:
    #             f.write(yaml.dump(self.app_config_yaml))

    def get_config_from_input_and_env(self, key: str, input_args):
        if input_args.get(key):
            return input_args.get(key)
        else:
            return self.get_config_from_env(key)

    def get_config_from_env(self, key: str):
        """
        input parameter > env variable > config file
        :param key:
        :return:
        """
        env_key = self.env_prefix + key.upper()
        yaml_lower_key = key.lower()
        yaml_upper_key = key.upper()
        if os.getenv(env_key):
            return os.getenv(env_key)
        elif self.app_config_yaml.get(yaml_upper_key):
            return self.app_config_yaml.get(yaml_upper_key)
        elif self.app_config_yaml.get(yaml_lower_key):
            return self.app_config_yaml.get(yaml_lower_key)
        else:
            return None

    def get_version_from_package(self):
        try:
            pkg_version = version(self.pkg_name)
        except PackageNotFoundError:
            pkg_version = "0.x"
        return pkg_version

    def get_data_from_yaml_string(self, yaml_string, default_content={}):
        try:
            return yaml.safe_load(yaml_string)
        except Exception as e:
            self.app_logger.tab_failure("Failed to read yaml string : {}".format(e))
            return default_content

    def get_data_from_yaml_file(self, yaml_file, default_content={}):
        if not os.path.exists(yaml_file):
            self.app_logger.tab_warning("{} not found".format(yaml_file))
            return default_content
        try:
            with open(yaml_file, "r", encoding="utf-8") as f:
                return yaml.safe_load(f.read())
        except Exception as e:
            self.app_logger.tab_failure("Failed to read {} : {}".format(yaml_file, e))
            return default_content

    def get_data_from_json_file(self, json_file, default_content={}):
        if not os.path.exists(json_file):
            self.app_logger.tab_warning("{} not found".format(json_file))
            return default_content
        try:
            with open(json_file, "r", encoding="utf-8") as f:
                return json.load(f)
        except Exception as e:
            self.app_logger.tab_failure("Failed to read {} : {}".format(json_file, e))
            return default_content

    def write_data_to_yaml_file(self, file_path, file_content):
        file_dir = os.path.dirname(file_path)
        if file_dir and not os.path.exists(file_dir):
            os.makedirs(file_dir, exist_ok=True)
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(yaml.dump(file_content))
        return True
    
    def copy_file(self, src_path, dest_path):
        if not os.path.exists(src_path):
            self.app_logger.tab_failure("Source file {} not found".format(src_path))
            return False
        dest_dir = os.path.dirname(dest_path)
        if dest_dir and not os.path.exists(dest_dir):
            os.makedirs(dest_dir, exist_ok=True)
        with open(src_path, "r", encoding="utf-8") as f_src:
            with open(dest_path, "w", encoding="utf-8") as f_dest:
                f_dest.write(f_src.read())
        return True


if __name__ == "__main__":
    print("🚀 This is a config package")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com


from datetime import datetime


class AppLogger(object):

    @staticmethod
    def exit():
        exit(1)

    @staticmethod
    def launch(log_content):
        """
        🏁
        :param log_content:
        :return:
        """
        print("\n\U0001F3C1 {}".format(log_content))
        return True

    @staticmethod
    def tab_launch(log_content):
        """
        ➤
        :param log_content:
        :return:
        """
        print("    \u27A4 {}".format(log_content))
        return True

    @staticmethod
    def failure(log_content):
        """
        ✘
        :param log_content:
        :return:
        """
        print("\n\u2718 {}".format(log_content))
        return False

    @staticmethod
    def tab_failure(log_content, tab_string="    "):
        """
        ✘
        :param log_content:
        :param tab_string:
        :return:
        """
        print("{}\u2718 {}".format(tab_string, log_content))
        return False

    @staticmethod
    def warning(log_content):
        """
        ❕
        :param log_content:
        :return:
        """
        print("\n\u2755 {}".format(log_content))
        return True

    @staticmethod
    def tab_warning(log_content, tab_string="    "):
        """
        ❕
        :param log_content:
        :param tab_string:
        :return:
        """
        print("{}\u2755 {}".format(tab_string, log_content))
        return True

    @staticmethod
    def success(log_content):
        """
        ✔
        :param log_content:
        :return:
        """
        print("\n\u2714 {}".format(log_content))
        return True

    @staticmethod
    def tab_success(log_content, tab_string="    "):
        """
        ✔
        :param log_content:
        :param tab_string:
        :return:
        """
        print("{}\u2714 {}".format(tab_string, log_content))
        return True

    @staticmethod
    def logo():
        """
        Generated by https://textkool.com/en/ascii-art-generator
        Font: Small Slant
        :return:
        """
        # exec_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        """
        """
        print("\n".join([
            "  ____  __       _    ___             ___               __    ",
            " / __ \/ /  ___ (_)__/ (_)__ ____    / _ | ___ ___ ___ / /____",
            "/ /_/ / _ \(_-</ / _  / / _ `/ _ \  / __ |(_-<(_-</ -_) __(_-<",
            "\____/_.__/___/_/\_,_/_/\_,_/_//_/ /_/ |_/___/___/\__/\__/___/",
            "                                         Designed by WILL     ",
            ""
        ]))
        return True

    @staticmethod
    def print_relative_time(time_string):
        time_object = datetime.strptime(str(time_string).split(".")[0], '%Y-%m-%dT%H:%M:%S')
        time_delta = datetime.utcnow() - time_object
        if time_delta.days > 0:
            time_unit = int(time_delta.days / 365)
            if time_unit > 0:
                return "{} years ago".format(time_unit)
            time_unit = int(time_delta.days / 30)
            if time_unit > 0:
                return "{} months ago".format(time_unit)
            return "{} days ago".format(time_delta.days)
        else:
            time_unit = int(time_delta.seconds / 3600)
            if time_unit > 0:
                return "{} hours ago".format(time_unit)
            time_unit = int(time_delta.seconds / 60)
            if time_unit > 0:
                return "{} minutes ago".format(time_unit)
            return "{} seconds ago".format(time_delta.seconds)


if __name__ == "__main__":
    print("🚀 This is a logger package")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com


import sys
from obsiasset.cli.appCli import AppParser
from obsiasset.cli.appApi import AppApi
from obsiasset.utils.appLogger import AppLogger


def main():
    app_cli_parser = AppParser()
    parser = app_cli_parser.parser

    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    args = parser.parse_args()
    args_cmd = args.command.replace("-", "_")

    AppLogger.logo()
    app_cli = AppApi(app_args=args.__dict__)
    try:
        app_cmd = getattr(app_cli, args_cmd)
        app_cmd()
        return True
    except AttributeError as e:
        AppLogger.failure(e)
        AppLogger.failure("Oops, some failures during execute command [{}]".format(args_cmd))
        return False


if __name__ == "__main__":
    print("🚀 This is main script")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com


import os
from jinja2 import Environment as j2Environment
from jinja2 import FileSystemLoader as j2FileSystemLoader
# from urllib.parse import urlparse, urlunparse
# import importlib.util
# from importlib.metadata import version, PackageNotFoundError
from obsiasset.utils.appLogger import AppLogger
from obsiasset.utils.appConfig import AppConfig


class AppManager(object):
    def __init__(self):
        self.app_config = AppConfig()
        self.app_logger = AppLogger()
        self.j2env_schemas = j2Environment(loader=j2FileSystemLoader(self.app_config.schema_dir))
        self.j2env_templates = j2Environment(loader=j2FileSystemLoader(self.app_config.tmpl_dir))
        self.j2env_samples = j2Environment(loader=j2FileSystemLoader(self.app_config.sample_dir))

    def get_supported_schemas(self):
        app_schemas = list()
        for item in os.listdir(self.app_config.schema_dir):
            if item.endswith(".yaml.j2"):
                app_schemas.append(item.replace(".yaml.j2", ""))
                # app_schema_file = os.path.join(self.app_config.schema_dir, item)
                # app_schema_yaml = self.app_config.get_data_from_yaml_file(app_schema_file)
                # if app_schema_yaml.get("name"):
                #     app_schemas.append(app_schema_yaml.get("name"))
        return app_schemas
    
    def get_supported_languages(self):
        app_languages = list()
        for item in os.listdir(self.app_config.i18n_dir):
            if item.endswith(".yaml"):
                lang_code = item.replace(".yaml", "").split("_")[-1]
                app_languages.append(lang_code)
        return app_languages

    def get_i18n_by_name(self, schema_name: str, lang_code: str):
        i18n_file_path = os.path.join(self.app_config.i18n_dir, "{}_{}.yaml".format(schema_name, lang_code))
        if not os.path.exists(i18n_file_path):
            self.app_logger.tab_failure("Cannot find i18n file by schema [{}] and language [{}]".format(
                schema_name,
                lang_code
            ))
            return None
        self.app_logger.tab_success("Load i18n file: {}".format(os.path.basename(i18n_file_path)))
        return self.app_config.get_data_from_yaml_file(i18n_file_path)

    def get_template_by_path(self, template_path: list):
        tmpl_file_path = os.path.join(self.app_config.tmpl_dir, *template_path)
        if not os.path.exists(tmpl_file_path):
            return None
        return self.app_config.get_data_from_yaml_file(tmpl_file_path)
    
    def setup_schema(self, schema_name: str, lang_code: str, abs_vault_path: list, import_sample: bool = False):
        i18n_config = self.get_i18n_by_name(schema_name, lang_code)
        if i18n_config is None:
            return False
        schema_config_string = self.j2env_schemas.get_template("{}.yaml.j2".format(schema_name)).render(i18n_config)
        schema_config = self.app_config.get_data_from_yaml_string(schema_config_string)
        sample_config = i18n_config.get("sample", {})
        for item in schema_config.get("entities", []):
            asset_path = item.get("path", [])
            template_path = item.get("template", [])
            abs_asset_path = os.path.join(abs_vault_path, *asset_path)
            if len(template_path) > 0:
                self.app_logger.tab_launch("Create file: {}".format(abs_asset_path))
                tmpl_content = self.j2env_templates.get_template(os.path.join(*template_path)).render(i18n_config)
                with open(abs_asset_path, "w", encoding="utf-8") as f:
                    f.write(tmpl_content)
                self.app_logger.tab_success("Done")
            else:
                self.app_logger.tab_launch("Create folder: {}".format(abs_asset_path))
                os.makedirs(abs_asset_path, exist_ok=True)
                self.app_logger.tab_success("Done")
            # Check and import sample assets
            sample_path = item.get("sample", [])
            sample_key = item.get("sample_key")
            if import_sample and len(sample_path) > 0 and sample_key and sample_key != "":
                for sample_item in sample_config.get(sample_key, []):
                    i18n_config['sample'][sample_key] = sample_item 
                    sample_name = "{}.md".format(sample_item.get("name", "?"))
                    abs_sample_path = os.path.join(abs_asset_path, sample_name)
                    self.app_logger.tab_launch("Import sample asset: {}".format(abs_sample_path))
                    sample_content = self.j2env_samples.get_template(os.path.join(*sample_path)).render(i18n_config)
                    with open(abs_sample_path, "w", encoding="utf-8") as f:
                        f.write(sample_content)
                    self.app_logger.tab_success("Done")
        return True


if __name__ == "__main__":
    print("🚀 This is a manager package")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: bxwill.shi@gmail.com


import os
# import shutil
import sys
# import requests
# import yaml
# from tabulate import tabulate
from obsiasset.utils.appLogger import AppLogger
from obsiasset.utils.appConfig import AppConfig
from obsiasset.utils.appManager import AppManager
# from crontab import CronTab
# from crontab import CronItem


def print_tree(start_path, indent=""):
    if indent == "":
        print(os.path.basename(start_path))
    for idx, item in enumerate(os.listdir(start_path)) if os.path.isdir(start_path) else {}:
        path = os.path.join(start_path, item)
        connector = "├── " if idx < len(os.listdir(start_path)) - 1 else "└── "
        print(indent + connector + item)
        if os.path.isdir(path):
            new_indent = indent + ("│   " if idx < len(os.listdir(start_path)) - 1 else "    ")
            print_tree(path, new_indent)
    return True



class AppApi(object):
    def __init__(self, app_args):
        self.app_config = AppConfig()
        self.app_args = app_args
        self.app_logger = AppLogger()
        self.app_manager = AppManager()

    def show_version(self):
        pkg_version = self.app_config.get_version_from_package()
        version_str_install = "{} [{}] version: {}".format(
            self.app_config.cmd_name,
            self.app_config.pkg_name,
            pkg_version
        )
        version_str_python = "Python version: {}".format(sys.version)
        version_str_tag = "\u00AF" * max(
            len(version_str_install),
            len(version_str_python.split("\n")[0])
        )
        print("\n".join([
            version_str_install,
            version_str_tag,
            version_str_python,
            version_str_tag,
        ]))
        return True

    def show_variables(self):
        # opt_items = OptItems()
        # for k, v in opt_items.__dict__.items():
        #     if k == "config_overwrite":
        #         continue
        #     print("\t".join([
        #         self.app_config.env_prefix + k.upper(),
        #         "{} : {}".format(v.help, " | ".join(v.choices)) if v.choices else v.help
        #     ]))
        return True

    def show_config(self):
        for k, v in self.app_config.app_config_yaml.items():
            if "_token" in k or "_secret" in k:
                self.app_logger.tab_success("{}: ******".format(k))
            else:
                self.app_logger.tab_success("{}: {}".format(k, v))
        return True

    def config_cred(self):
        config_json = dict()
        for key, value in self.app_args.items():
            if key not in ["command", "overwrite"] and value:
                config_json[key] = value
        self.app_config.create_config_file(
            config_file_json=config_json,
            force=True
        )
        self.app_logger.success("Create/Update config file:")
        self.show_config()
        return True

    def init(self):
        # param_vault = str(self.app_args.get("vault")).lower()
        param_schema = str(self.app_args.get("schema")).lower()
        param_lang = str(self.app_args.get("lang")).lower()
        param_sample = self.app_args.get("import_sample")
        self.app_logger.launch("Start to init a vault folder structure by schema [{}] and language [{}]...".format(
            param_schema,
            param_lang
        ))

        # if param_vault is None or param_vault == "":
        #     param_vault = "."
        # vault_path = os.path.join(
        #     self.app_config.app_workdir,
        #     param_vault
        # )
        setup_result = self.app_manager.setup_schema(
            schema_name=param_schema,
            lang_code=param_lang,
            abs_vault_path=self.app_config.app_workdir,
            import_sample=param_sample
        )
        if setup_result:
            self.app_logger.success("Complete to init vault folder structure.")
        else:
            self.app_logger.failure("Fail to init vault folder structure.")
            return False
        # if param_sample:
        #     import_result = self.app_manager.import_sample_assets(
        #         # self.app_config.app_workdir
        #     )
        #     if import_result:
        #         self.app_logger.success("Complete to import sample assets.")
        #     else:
        #         self.app_logger.failure("Fail to import sample assets.")
        #         return False
        self.app_logger.success("🎉 All done!")
        return True

    def reorg(self):
        pass


if __name__ == "__main__":
    print("🚀 This is an API package")